@@ -27,6 +27,7 @@
 #include <hildon/hildon.h>
 #include <libosso.h>
 
+
 #ifdef WITH_SHARING
 
 #ifdef WITH_SHARING_H
@@ -49,25 +50,46 @@ GtkWidget *dw;
 GtkWidget *source_button, *sharing_button, *format_button;
 Dicto *d;
 GtkTreeModel *formats;
+DictoFormats f=FORMAT_WAV_8K_16B_M;
 
 #define MYDICTO_GCONF_PATH "/apps/mydicto"
 #define MYDICTO_GCONF_AUDIO_FORMAT MYDICTO_GCONF_PATH  "/audio-format"
 
+#define PHONE_SRC "liveadder name=phonemixer ! pulsesrc device=sink.hw0.monitor ! phonemixer. pulsesrc device=source.hw0 ! phonemixer."
+
+static void
+mydicto_gconf_store(DictoFormats f)
+{
+GConfClient *client;
+
+client=gconf_client_get_default();
+gconf_client_set_int(client, MYDICTO_GCONF_AUDIO_FORMAT, f, NULL);
+}
+
 static gboolean
-get_selected_id_from_selector(HildonTouchSelector *selector, gint *id)
+get_selected_id_from_selector(HildonTouchSelector *selector, gint *id, gchar **desc)
 {
 GtkTreePath *path;
 GtkTreeModel *model;
 GtkTreeIter iter;
 
+#if 0
 path=hildon_touch_selector_get_last_activated_row(selector, 0);
-if (path!=NULL) {
-	model=hildon_touch_selector_get_model(selector, 0);
-	gtk_tree_model_get_iter(model, &iter, path);
-	gtk_tree_model_get(model, &iter, 0, id, -1);
-}
+if (path==NULL)
+	return FALSE;
+model=hildon_touch_selector_get_model(selector, 0);
+gtk_tree_model_get_iter(model, &iter, path);
+gtk_tree_model_get(model, &iter, 0, id, 1, desc, -1);
 gtk_tree_path_free(path);
-return path ? TRUE : FALSE;
+#else
+if (!hildon_touch_selector_get_selected(selector, 0, &iter))
+	return FALSE;
+
+model=hildon_touch_selector_get_model(selector, 0);
+gtk_tree_model_get(model, &iter, 0, id, 1, desc, -1);
+#endif
+
+return TRUE;
 }
 
 static gint
@@ -79,10 +101,16 @@ return OSSO_OK;
 static gboolean
 dicto_error_cb(Dicto *d, gpointer err, gpointer data)
 {
+gchar *msg;
+GtkWidget *info;
 GError *e=(GError *)err;
 
 g_debug("Dicto reported error: (%d) %s", e->code, e->message);
-hildon_banner_show_information(GTK_WIDGET(window), "", e->message);
+
+msg=g_strdup_printf("Error:\n%s", e->message);
+info=hildon_note_new_information(GTK_WINDOW(window), msg);
+gtk_dialog_run(GTK_DIALOG(info));
+g_free(msg);
 return FALSE;
 }
 
@@ -120,9 +148,22 @@ return FALSE;
 }
 
 static void
-mydicto_source_cb(GtkWidget *w, gpointer data)
+mydicto_source_selection_cb(HildonTouchSelector *selector, gint c, gpointer data)
 {
+gint srcid;
 
+srcid=hildon_touch_selector_get_active(selector, 0);
+g_debug("Src: %d", srcid);
+}
+
+static void
+mydicto_format_selection_cb(HildonTouchSelector *selector, gint c, gpointer data)
+{
+gint id;
+gchar *desc, *msg;
+
+if (!get_selected_id_from_selector(selector, &id, &desc))
+	return;
 }
 
 static void
@@ -130,11 +171,18 @@ mydicto_format_cb(HildonPickerButton *button, gpointer data)
 {
 HildonTouchSelector *selector=(HildonTouchSelector *)data;
 gint id;
+gchar *desc, *msg;
 
-if (!get_selected_id_from_selector(selector, &id))
+if (!get_selected_id_from_selector(selector, &id, &desc))
 	return;
-if (!dicto_set_format(d, (DictoFormats)id))
+if (!dicto_set_format(d, (DictoFormats)id)) {
 	hildon_banner_show_information(GTK_WIDGET(window), "", "Failed to set new format");
+	return;
+}
+msg=g_strdup_printf("Using format:\n%s", desc);
+hildon_banner_show_information(GTK_WIDGET(window), "", msg);
+g_free(msg);
+mydicto_gconf_store(id);
 }
 
 #if HILDON_CHECK_VERSION(2,2,0)
@@ -145,7 +193,6 @@ DictoWidget *dw=(DictoWidget *)data;
 gchar *file;
 #ifdef WITH_SHARING
 g_object_get(G_OBJECT(d), "current-file", &file, NULL);
-g_debug("File: %s", file);
 if (file && strlen(file)>0 && g_file_test(file, G_FILE_TEST_IS_REGULAR)) {
 	sharing_dialog_with_file(osso, GTK_WINDOW(window), file);
 } else {
@@ -173,7 +220,7 @@ static HildonAppMenu *
 mydicto_menu(void)
 {
 HildonAppMenu *menu;
-HildonTouchSelector *s;
+HildonTouchSelector *f_selector, *src_selector;
 HildonTouchSelectorColumn *column;
 
 menu=HILDON_APP_MENU(hildon_app_menu_new());
@@ -184,22 +231,31 @@ g_signal_connect_after(sharing_button, "clicked", G_CALLBACK(mydicto_menu_button
 hildon_app_menu_append(menu, GTK_BUTTON(sharing_button));
 
 #if 0
-source_button=gtk_button_new_with_label("Source");
-g_signal_connect_after(source_button, "clicked", G_CALLBACK(mydicto_source_cb), dw);
+source_button=hildon_picker_button_new(HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+hildon_button_set_title(HILDON_BUTTON(source_button), "Source");
+src_selector=hildon_touch_selector_new_text();
+hildon_touch_selector_append_text(src_selector, "Microphone");
+hildon_touch_selector_append_text(src_selector, "Phone");
+hildon_touch_selector_set_active(src_selector, 0, 0);
+g_signal_connect_after(src_selector, "changed", G_CALLBACK(mydicto_source_selection_cb), NULL);
+hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(source_button), HILDON_TOUCH_SELECTOR(src_selector));
 hildon_app_menu_append(menu, GTK_BUTTON(source_button));
 #endif
 
 format_button=hildon_picker_button_new(HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
 hildon_button_set_title(HILDON_BUTTON(format_button), "Format");
 
-s=hildon_touch_selector_new();
-column=hildon_touch_selector_append_column(s, GTK_TREE_MODEL(formats), gtk_cell_renderer_text_new(), "text", 1, NULL);
+f_selector=hildon_touch_selector_new();
+column=hildon_touch_selector_append_column(f_selector, GTK_TREE_MODEL(formats), gtk_cell_renderer_text_new(), "text", 1, NULL);
 hildon_touch_selector_column_set_text_column(column, 1);
-hildon_touch_selector_set_hildon_ui_mode(s, HILDON_UI_MODE_NORMAL);
-hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(format_button), HILDON_TOUCH_SELECTOR(s));
-hildon_app_menu_append(menu, GTK_BUTTON(format_button));
+hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(format_button), HILDON_TOUCH_SELECTOR(f_selector));
 
-g_signal_connect(G_OBJECT(format_button), "value-changed", G_CALLBACK(mydicto_format_cb), s);
+hildon_touch_selector_set_column_selection_mode(f_selector, HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
+
+g_signal_connect(G_OBJECT(format_button), "value-changed", G_CALLBACK(mydicto_format_cb), f_selector);
+g_signal_connect(G_OBJECT(f_selector), "changed", G_CALLBACK(mydicto_format_selection_cb), NULL);
+
+hildon_app_menu_append(menu, GTK_BUTTON(format_button));
 
 gtk_widget_show_all(GTK_WIDGET(menu));
 return menu;
@@ -226,7 +282,6 @@ return menu;
 static void
 dicto_format_select_cb(GtkComboBox *widget, gpointer data)
 {
-GConfClient *client;
 GtkTreeIter iter;
 GtkTreeModel *model;
 gint value=-1;
@@ -236,12 +291,8 @@ if (!gtk_combo_box_get_active_iter(widget, &iter))
 	return;
 
 gtk_tree_model_get(model, &iter, 0, &value,  -1);
-if (!dicto_set_format(d, value)) {
+if (!dicto_set_format(d, value))
 	hildon_banner_show_information(GTK_WIDGET(window), "", "Failed to set format");
-	return;
-}
-client=gconf_client_get_default();
-gconf_client_set_int(client, MYDICTO_GCONF_AUDIO_FORMAT, value, NULL);
 }
 
 static gboolean
@@ -300,7 +351,6 @@ main(gint argc, gchar * argv[])
 {
 gchar *sink, *src;
 GConfClient *client;
-DictoFormats f=FORMAT_WAV_8K_16B_M;
 
 #if HILDON_CHECK_VERSION(2,2,0)
 hildon_gtk_init(&argc, &argv);
@@ -340,7 +390,6 @@ if (argc>4) {
 	g_debug("Pulseaudio role set to: %s", argv[4]);
 }
 
-
 if (!mydicto_init(f, src, sink)) {
 	g_warning("myDicto init failed");
 	return 1;
